#!/usr/bin/python2
#
# Output snarf alerts from the DNS pipeline mediator in IODEF format
# ------------------------------------------------------------------------
# Copyright (C) 2012 Carnegie Mellon University. All Rights Reserved.
# ------------------------------------------------------------------------
# Authors: Tony Cebzanov <tonyc@cert.org>
# -----------------------------------------------------------------------------
# GNU Public License (GPL) Rights pursuant to Version 2, June 1991
# Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
# -----------------------------------------------------------------------------
#
# Sample Usage:
#
#     snarf2iodef.py tcp://127.0.0.1:5556


import signal
from optparse import OptionParser
from snarf import *
from snarf.alert import datetime_from_timestamp
import netsa_silk as ns
import uuid
import os

import xml.etree.cElementTree as ET
from xml.dom.minidom import parseString
import functools

# Explicitly handle SIGINT and raise KeyboardInterrupt so that
# it's not handled by another thread
def sigint_handler(signum, frame):
    raise KeyboardInterrupt


class _E(object):

    def __call__(self, tag, *children, **attrib):
        elem = ET.Element(tag, attrib)
        for item in children:
            if isinstance(item, dict):
                elem.attrib.update(item)
            elif isinstance(item, basestring):
                if len(elem):
                    elem[-1].tail = (elem[-1].tail or "") + item
                else:
                    elem.text = (elem.text or "") + item
            elif isinstance(item, type(None)):
                pass
            elif ET.iselement(item):
                elem.append(item)
            else:
                raise TypeError("bad argument: %r" % item)
        return elem

    def __getattr__(self, tag):
        return functools.partial(self, tag)

# create factory object
E = _E()

def TYPE(*args):
    # type is a reserved word, so we use a helper function
    return {"type": " ".join(args)}

def prettify(element, level=0, indent=4):
    txt = ET.tostring(element, encoding="UTF-8")
    return parseString(txt).toprettyxml(indent=" " * indent,
                                        newl="\n" + " " * (level * indent),
                                        encoding="UTF-8")



def alert_to_incidents(alert):

    incidents = list()
    tags = ','.join(alert.get_tags())

    for flow in alert.get_fields()['flow']:

        incidentid = str(uuid.uuid4())

        def address_to_system(addr, port, proto, label):
            return E.System(
                E.Node(
                    E.Address(str(addr),category =
                              (addr.is_ipv6()
                               and "ipv6-addr"
                               or "ipv4-addr"
                               )
                              ),
                    E.Service(E.Port(str(port)),
                              ip_protocol=str(proto))
                    ),
                category=label
                )

        sip = flow.sip
        dip = flow.dip
        sport = flow.sport
        dport = flow.dport
        proto = flow.proto

        if options.name:
            incidentidxml = E.IncidentID(incidentid, name=options.name)
        else:
            incidentidxml = E.IncidentID(incidentid)

        incidentxml = E.Incident(
            incidentidxml,
            E.Description(tags),
            E.EventData(
                E.Flow(
                    address_to_system(sip, sport, proto, "source"),
                    address_to_system(dip, dport, proto, "target"),
                    )
                )
            )
        incidents.append([incidentid, incidentxml])

    return incidents

def handle_sink(alert):

    for (incidentid, incidentxml) in alert_to_incidents(alert):
        iodef_doc = ET.Element(
            "IODEF-Document",
            {"version": "1.00",
             "lang": "en",
             "formatid": "watchlist",
             "xmlns": "urn:ietf:params:xml:ns:iodef-1.0",
             "xmlns:xsi": "http://www.w3.org/2001/XMLSchema-instance",
             "xsi:schemaLocation": "urn:ietf:params:xml:schema:iodef-1.0"
             })
        filename = "%s.xml" %(incidentid)
        print filename
        outfile = open(os.path.join(options.outdir, filename), "w")
        iodef_doc.append(incidentxml)
        print
        print >> outfile, prettify(iodef_doc)
        outfile.close()

def main():
    global options
    parser = OptionParser()
    parser.add_option("-c", "--channel", dest="channel",
                      help="alert channel to subscribe to",
                      default=None)
    parser.add_option("-o", "--output-directory", dest="outdir",
                      default="snarf2iodef-out", help="output directory"),
    parser.add_option("-n", "--name", dest="name", help="domain name")
    (options, args) = parser.parse_args()

    if not os.path.exists(options.outdir):
        os.makedirs(options.outdir)

    if len(args) < 1:
        raise Exception("usage: %s socket" %(sys.argv[0]))
    socket=args[0]

    signal.signal(signal.SIGINT, sigint_handler)

    try:
        sink = Sink(handle_sink, origin=socket, channel=options.channel)
        while True:
            sink.get_thread().join(1000)
            if not sink.get_thread().isAlive:
                break
    except KeyboardInterrupt:
        sink.shutdown()


if __name__ == '__main__':
    main()
